# ensemble_hamilton.spec
# Specification algebrique des ensembles d'entiers de 1 a n
# pour le projet sur les chemins hamiltoniens
# Auteur : VOURIOT Thierry
# Date : 21 novembre 2001

# Sorte S definissant l'egalite
spec EG etend BASE

	sorte S

	operations

		==: S S -> Bool		#Egalite
		!=: S S -> Bool		#Difference

	axiomes

		(x==x) = vrai
		(x!=y) = non (x==y)

fspec

spec BSUP	
	operations	
		n: -> Nat
fspec

spec ENSEMBLE(EG,BSUP) etend BASE

	sorte Ens

	operations

		ensnouv: -> Ens			#Ensemble vide		
		ins: Ens Nat -> Ens		#Insertion d'un entier		
		sing: Nat -> Ens		#Ensemble avec un entier		
		supp: Ens Nat -> Ens		#Suppression d'un entier		
		app: Ens Nat -> Bool		#Appartenance d'un entier a l'ensemble		
		union: Ens Ens -> Ens		#Union de 2 ensembles	
		inter: Ens Ens -> Ens		#Intersection de 2 ensembles		
		ref: -> Ens			#Ensemble "plein"
		card: Ens -> Nat		#Cardinalite (nb d'elements)
		vide: Ens -> Bool		#Ensemble est vide ?
		inclu: Ens Ens -> Bool		#Inclusion d'un ensemble dans un autre
		eg: Ens Ens -> Bool		#Egalite de 2 ensembles
		comp: Ens -> Ens		#Complementaire
		diff: Ens Ens -> Ens		#Difference entre 2 ensembles
		copie: Ens -> Ens		#Copie d'un ensemble
		aff: Ens ->			#Affiche l'ensemble  l'ecran
		lirens: -> Ens			#Lit un ensemble au clavier


	preconditions

		pre ins(e,x) = 1<=x<=n et non app(e,x)
		pre sing(x) = 1<=x<=n
		pre supp(e,x) = 1<=x<=n et app(e,x) 
		pre app(x) = 1<=x<=n


	axiomes
		#Constructeurs ensnouv et ins

		supp(ins(e,x),y) = si x==y alors e
				sinon ins(supp(e,y),x) fsi
		
		app(ensnouv,x) = faux
		app(ins(e,x),y) = si x==y alors vrai
				sinon app(e,x) fsi
						
		sing(x) = ins(ensnouv,x)

		union(ensnouv,e2) = e2
		union(ins(e1,x1),e2) = si app(x1,e2) alors union(e1,e2) 
					sinon ins(union(e1,e2),x1) fsi

		inter(ensnouv,e2) = ensnouv
		inter(ins(e1,x1),e2) = si non app(x1,e2) alors inter(e1,e2) 
					sinon ins(inter(e1,e2),x1) fsi

		ref = e avec (e,i) = init(ensnouv,1)
			tantque i<=N
			rep (ins(e,i),i+1) frep

		card(ensnouv) = 0
		card(ins(e,x)) = card(e) + 1

		vide(ensnouv) = vrai
		vide(ins(e,x)) = faux

		inclu(e1,ensnouv) = vrai
		inclu(e1,ins(e2,x2)) = si app(e1,x2) alors inclu(e1,e2)
					sinon faux fsi

		eg(e1,e2) = inclu(e1,e2) et inclu(e2,e1)

		comp(ensnouv) = ref
		comp(ins(e,x)) = supp(comp(e),x)

		diff(e1,ensnouv) = e1
		diff(ensnouv,e2) = ensnouv
		diff(ins(e1,x),e2) = si app(x,e2) alors diff(e1,e2)
				sinon ins(diff(e1,e2),x) fsi

		copie(ensnouv) = ensnouv
		copie(ins(e,x)) = ins(e,x)

		aff(e) = avec i = init 1
			tantque i <= N
			rep si app(e,i) ecrire i fsi frep
		
		lirens = e avec (e,x) = init(ensnouv,donnee)
			tantque x > 0 
			rep si 1<=x<=N et non app(e,x)
				alors (ins(e,x),donnee) fsi
			frep

fspec
