# graphe_hamilton.spec
# Specification algebrique des graphes pour
# le projet sur les chemins hamiltoniens
# Auteur : VOURIOT Thierry
# Date : 29 novembre 2001

# Sorte S definissant l'egalite
spec EG etend BASE

	sorte S

	operations

		==: S S -> Bool		#Egalite
		!=: S S -> Bool		#Difference

	axiomes

		(x==x) = vrai
		(x!=y) = non (x==y)

fspec

# Definition de la sorte avec les operations "standards"
spec GRAPHE(EG)

	sorte Graphe

	operations

		grnouv: -> Graphe		#Creation du graphe vide
		adjs: Graphe S -> Graphe	#Adjonction d'un sommet
		adja: Graphe S S -> Graphe	#Adjonction d'un arc
		supps: Graphe S -> Graphe	#Suppression d'un sommet
		suppa: Graphe S S -> Graphe	#Suppression d'un arc
		di: Graphe S -> Nat		#Degre interieur
		de: Graphe S -> Nat		#Degre exterieur
		exs: Graphe S -> Bool		#Existance d'un sommet
		exa: Graphe S S -> Bool		#Existance d'un arc


	preconditions 

		pre adjs(g,x) = non exs(g,x)
		pre adja(g,x,y) = exs(g,x) et exs(g,y) et non exa(g,x,y)
		pre supps(g,x) = exs(g,x) et di(g,x)==0 et de(g,x)==0
		pre suppa(g,x,y) = exs(g,x) et exs(g,y) et exa(g,x,y)
		pre di(g,x) = pre de(g,x) = exs(g,x)
		pre exa(g,x,y) = exs(g,x) et exs(g,y)

		
	axiomes 
		#Constructeurs grnouv adjs et adja

		exs(grnouv,z) = faux
		exs(adjs(g,x),z) = x==z ou exs(g,z)
		exs(adja(g,x,y),z) = exs(g,z)

		exa(grnouv,z,t) = faux
		exa(adjs(g,x),z,t) = exa(g,z,t)
		exa(adja(g,x,y),z,t) = (x,y)==(z,t) ou exa(g,z,t)

		di(adjs(g,x),z) = si x==z alors 0
				sinon di(g,z) fsi
		di(adja(g,x,y),z) = si y==z alors di(g,z)+1
				sinon di(g,z) fsi

		de(adjs(g,x),z) = si x==z alors 0
				sinon de(g,z) fsi
		de(adja(g,x,y),z) = si x==z alors de(g,z)+1
				sinon de(g,z) fsi

		supps(adjs(g,x),z) = si x==z alors g
				sinon adjs(supps(g,z),x) fsi
		supps(adja(g,x,y),z) = adja(supps(g,z),x,y)

		suppa(adjs(g,x),z,t) = adjs(suppa(g,z,t),x)
		suppa(adja(g,x,y),z,t) = si (x,y)==(z,t) alors g
				sinon adja(suppa(g,z,t),x,y) fsi
		
fspec

# Ajout d'operations sur les graphes
spec GRAPHE1(EG) etend GRAPHE(EG),ENSEMBLE(EG,BSUP)

	operations

		es: Graphe -> Ens		#Ensemble des sommets
		esucc: Graphe S -> Ens		#Ensemble des successeurs d'un sommet
		epred: Graphe S -> Ens		#Ensemble des predecesseurs d'un sommet


	preconditions 

		pre esucc(g,x) = pre epred(g,x) = exs(g,x)
		

	axiomes 
		#Constructeurs grnouv adjs et adja

		es(grnouv) = ensnouv
		es(adjs(g,x)) = ins(es(g),x)
		es(adja(g,x,y)) = es(g)

		esucc(adjs(g,x),z) = si x==z alors ensnouv
				sinon esucc(g,z) fsi
		esucc(adja(g,x,y),z) = si z==x alors ins(esucc(g,z),y)
					sinon esucc(g,z) fsi

		epred(adjs(g,x),z) = si x==z alors ensnouv
				sinon epred(g,z) fsi
		epred(adja(g,x,y),z) = si z==y alors ins(epred(g,z),x)
					sinon epred(g,z) fsi
	
fspec

# Ajout d'operations sur les graphes pour le projet
spec GRAPHE_H(EG) etend GRAPHE1(EG),LISTE(EG)

	operations

		ls: Graphe -> Liste		#Liste des sommets
		complet: Graphe -> Bool		#Graphe complet
		centre: Graphe Ens -> S		#Premier sommet de demi-degre exterieur max
						#appartenant  un ensemble
		suppsa: Graphe S -> Graphe	#Suppression d'un sommet et tous
						#les arcs qui y partent et arrivent
		hamilton: Graphe -> Liste	#Chemin hamiltonien
		

	preconditions 

		pre complet(g) = g!=grnouv
		pre centre(g) = g!=grnouv et !vide(Ens)
		pre suppsa(g,x) = exs(g,x)
		pre hamilton(g) = complet(g)


	axiomes 
		#Constructeurs grnouv adjs et adja

		ls(grnouv) = listenouv
		ls(adjs(g,x)) = adjq(ls(g),x)
		ls(adja(g,x,y)) = ls(g)

		complet(g) = res
			avec (l,res,i) = init(ls(g),vrai,1)
			tantque non vide(l) et res==vrai
			rep tantque i < lgr(l) et res==vrai
				rep (l,si exa(g,elem(l,i),tete(l) 
					ou exa(g,tete(l),elem(l,i))
					alors vrai else faux fsi,i+1)
				frep
				(suppt(l),res,1)
			frep

		centre(g,e) = c
			avec l = ls(g)
			avec (c,i) = init(tete(l),1)
			tantque i < lgr(l)
			rep avec tmp = elem(i)
				si app(e,tmp) 
					alors si !app(e,c) ou de(g,tmp) > de(g,c)
						alors (tmp,i+1)
						sinon (c,i+1)
						fsi
				fsi
			frep

		suppsa(adjs(g,x),z) = si x==z alors g
					sinon adjs(suppsa(g,z),x) fsi
		suppsa(adja(g,x,y),z) = si z==x ou z==y alors suppsa(g,z)
					sinon adja(suppsa(g,z),x,y) fsi

		hamilton(g) = l
			avec (c,g1,l) = init(centre(g,es(g)),g,adjq(listenouv,c))
			tantque (!vide(es(g1)))
			rep avec e = esucc(g1,c)
				(centre(g1,e),suppsa(g1,c),adjq(l,c))
			frep

		
fspec

