/* graphe_ma.c
 * Fichier source pour l'implantation des graphes 
 * d'entiers sous la forme de matrices d'adjacences
 * Auteur : VOURIOT Thierry
 * Date : 7 janvier 2002
 */

#include "graphe_ma.h"

/* Creation du graphe vide */
Graphe grnouv(void)
{
	int i,j;
	Graphe g = MALLOC(SGraphe);
	g->arcs = MALLOCN(Bool,N*N);
	g->sommets = ensnouv();

	for (i=0;i<N;i++)
		for (j=0;j<N;j++)
			g->arcs[i*N+j] = false;
	
	return g;
}

/* Adjonction d'un sommet
 * pre adjs(g,x) = non exs(g,x) et 0<=x<=N-1
 */
Graphe adjs(Graphe g, Nat s)
{
	g->sommets = ins(g->sommets,s);
	return g;
}

/* Adjonction d'un arc
 * pre adja(g,x,y) = exs(g,x) et exs(g,y) et non exa(g,x,y)
 */
Graphe adja(Graphe g, Nat x, Nat y)
{
	g->arcs[x*N+y] = true;
	return g;
}

/* Suppression d'un sommet
 * pre supps(g,x) = exs(g,x) et di(g,x)==0 et de(g,x)==0
 */
Graphe supps(Graphe g, Nat s)
{
	g->sommets = supp(g->sommets,s);
	return g;
}

/* Suppression d'un arc
 * pre suppa(g,x,y) = exs(g,x) et exs(g,y) et exa(g,x,y)
 */
Graphe suppa(Graphe g, Nat x, Nat y)
{
	g->arcs[x*N+y] = false;
	return g;
}

/* Degre interieur
 * pre di(g,x) = exs(g,x)
 */
Nat di(Graphe g, Nat x)
{
	int i,r=0;
	for (i=0;i<N;i++)
		if (g->arcs[i*N+x])
			r++;
	return r;
}

/* Degre exterieur
 * pre de(g,x) = exs(g,x)
 */
Nat de(Graphe g, Nat x)
{
	int i,r=0;
	for (i=0;i<N;i++)
		if (g->arcs[x*N+i])
			r++;
	return r;
}

/* Existance d'un sommet 
 * pre exs(g,x) = 0<=x<=N-1
 */
Bool exs(Graphe g, Nat s)
{
	return (app(g->sommets,s) == true);
}

/* Existance d'un arc 
 * pre exa(g,x,y) = exs(g,x) et exs(g,y)
 */
Bool exa(Graphe g, Nat x, Nat y)
{
	return (g->arcs[x*N+y] == true);
}

/* Ensemble des sommets */
Ens es(Graphe g)
{
	return g->sommets;
}

/* Ensemble des successeurs d'un sommet
 * pre esucc(g,x) = exs(g,x)
 */
Ens esucc(Graphe g, Nat s)
{
	Ens succ = ensnouv();
	int i;
	for (i=0;i<N;i++)
		if ((exa(g,s,i)) && exs(g,i))
			succ = ins(succ,i);
	return succ;
}

/* Ensemble des predecesseurs d'un sommet
 * pre epred(g,x) = exs(g,x)
 */
Ens epred(Graphe g, Nat s)
{
	Ens pred = ensnouv();
	int i;
	for (i=0;i<N;i++)
		if ((exa(g,i,s)) && exs(g,i))
			pred = ins(pred,i);
	return pred;
}

/* Affiche la matrice d'adjacences */
void affMat(Graphe g)
{
	int i,j;
	printf("  ");
	aff(g->sommets);
	for (i=0;i<N;i++)
		if (exs(g,i))
		{
			printf("%2d| ",i);
			for (j=0;j<N;j++)
			{
				if (exs(g,j))
					printf("%2d ",exa(g,i,j));
			}
			printf("\n");
		}
}

/* Copie */
Graphe cp(Graphe g)
{
	Graphe res = grnouv();
	int i,j;
	for (i=0;i<N*N;i++)
			res->arcs[i] = g->arcs[i];
	res->sommets = copie(g->sommets);
	return res;
}

/* Liste des sommets */
Liste ls(Graphe g)
{
	Liste res = listenouv();
	int i;
	for (i=0;i<N;i++)
		if (exs(g,i))
			res = adjq(res,i);
	return res;
}

/* Graphe complet */
Bool complet(Graphe g)
{
	Liste l = ls(g);
	Bool res = true;
	int i;
	while (!lvide(l) && (res==true))
	{
		i = 1;
		while (i<lgr(l) && (res==true))
		{
			if (exa(g,elem(l,i),tete(l)) || exa(g,tete(l),elem(l,i)))
				res = true;
			else res = false;
			i++;
		}
		l = suppt(l);
	}
	return res;
}

/* Premier sommet de demi-degre exterieur maximum appartenant  un ensemble
 * pre centre(g) = g!=grnouv et !vide(Ens)
 */
Nat centre(Graphe g, Ens e)
{
	Liste l = ls(g);
	Nat c = tete(l);
	int i = 1;
	Nat lg = lgr(l);
	Nat tmp;
	while (i<lg)
	{
		tmp = elem(l,i);
		
		if (app(e,tmp))
			if (!app(e,c) || (de(g,tmp) > de(g,c)))
				c = tmp;
		i++;
	}
	return c;
}
	
/* Suppression d'un sommet et de tous les arcs qui y partent et arrivent
 * pre suppsa(g,x) = exs(g,x)
 */
Graphe suppsa(Graphe g, Nat x)
{
	int i;
	/* suppression des arcs */
	for (i=0;i<N;i++)
	{
		if (g->arcs[i*N+x])
			g->arcs[i*N+x] = false;	/* arcs vers x */
		if (g->arcs[x*N+i])
			g->arcs[x*N+i] = false; /* arcs partant de x */
	}
	return supps(g,x);
}	

/* Chemin hamiltonien 
 * pre hamilton(g) = complet(g)
 */
Liste hamilton(Graphe g) 
{
	Graphe g1 = cp(g);
	Liste l = listenouv();
	Nat c;
	Ens e;

	c = centre(g1,es(g));
	e = esucc(g1,c);
	g1 = suppsa(g1,c);
	l = adjq(l,c);

	while (!vide(es(g1)))
	{
		c = centre(g1,e);
		e = esucc(g1,c);
		g1 = suppsa(g1,c);
		l = adjq(l,c);
	}
	free(g1);
	return l;
}
	
/* Programme de test */
/*int main()
{
	Graphe g = grnouv();
	g = adjs(adjs(adjs(adjs(g,1),3),5),7);
	g = adja(adja(adja(adja(adja(adja(adja(adja(g,1,3),1,5),7,1),3,5),7,3),5,7),5,3),7,5);
	affMat(g);
	printf("\nes = ");aff(es(g));
	printf("complet = %s\n",complet(g)?"vrai":"faux");
	printf("degre du centre = %d\n",de(g,centre(g)));
	printf("ls = ");laff(ls(g));
	printf("hamilton = ");laff(hamilton(g));
}*/
