/* graphe_ma.h
 * Fichier en tete pour l'implantation des graphes 
 * d'entiers de 1  N-1 sous la forme d'une matrice d'adjacence
 * Auteur : VOURIOT Thierry
 * Date : 5 janvier 2002
 */

#include "base.h"
#include "ens_tab.h"
#include "liste_ch.h"

/* Implantation sous la forme d'un ensemble de sommets
 * et d'une matrice d'adjacence pour les arcs 
 */
typedef struct sGraphe
{
	Ens sommets;
	Bool *arcs; /* Matrice -> un tableau de N*N */
}SGraphe, *Graphe;

/* Creation du graphe vide */
extern Graphe grnouv(void);

/* Adjonction d'un sommet
 * pre adjs(g,x) = non exs(g,x) et 0<=x<=N-1
 */
extern Graphe adjs(Graphe, Nat);

/* Adjonction d'un arc
 * pre adja(g,x,y) = exs(g,x) et exs(g,y) et non exa(g,x,y)
 */
extern Graphe adja(Graphe, Nat, Nat);

/* Suppression d'un sommet
 * pre supps(g,x) = exs(g,x) et di(g,x)==0 et de(g,x)==0
 */
extern Graphe supps(Graphe, Nat);

/* Suppression d'un arc
 * pre suppa(g,x,y) = exs(g,x) et exs(g,y) et exa(g,x,y)
 */
extern Graphe suppa(Graphe, Nat, Nat);

/* Degre interieur
 * pre di(g,x) = exs(g,x)
 */
extern Nat di(Graphe, Nat);

/* Degre exterieur
 * pre de(g,x) = exs(g,x)
 */
extern Nat de(Graphe, Nat);

/* Existance d'un sommet 
 * pre exs(g,x) = 0<=x<=N-1
 */
extern Bool exs(Graphe, Nat);

/* Existance d'un arc 
 * pre exa(g,x,y) = exs(g,x) et exs(g,y)
 */
extern Bool exa(Graphe, Nat, Nat);

/* Ensemble des sommets */
extern Ens es(Graphe);

/* Ensemble des successeurs d'un sommet
 * pre esucc(g,x) = exs(g,x)
 */
extern Ens esucc(Graphe, Nat);

/* Ensemble des predecesseurs d'un sommet
 * pre epred(g,x) = exs(g,x)
 */
extern Ens epred(Graphe, Nat);

/* Affiche la matrice d'adjacences */
extern void affMat(Graphe);

/* Copie */
extern Graphe cp(Graphe);

/* Liste des sommets */
extern Liste ls(Graphe);

/* Graphe complet
 * pre complet(g) = g!=grnouv
 */
extern Bool complet(Graphe);

/* Premier sommet de demi-degre exterieur maximum appartenant  un ensemble
 * pre centre(g) = g!=grnouv et !vide(Ens)
 */
extern Nat centre(Graphe, Ens);

/* Suppression d'un sommet et de tous les arcs qui y partent et arrivent
 * pre suppsa(g,x) = exs(g,x)
 */
extern Graphe suppsa(Graphe, Nat);

/* Chemin hamiltonien 
 * pre hamilton(g) = complet(g)
 */
extern Liste hamilton(Graphe);
